"""DSL operations subpackage.

Contains the individual operation implementations, split by domain.
Import specific operations from their submodules directly, e.g.
``from razin.dsl.operations.url_domain import run_url_domain_filter``.
"""
