"""Shared utility helpers."""

from __future__ import annotations

from .naming import normalize_similarity_name, sanitize_output_name

__all__ = ["normalize_similarity_name", "sanitize_output_name"]
