"""Constants for filesystem discovery and skill-name derivation."""

from __future__ import annotations

SKILL_MARKDOWN_FILENAME: str = "SKILL.md"
SKILL_NAME_FALLBACK: str = "unnamed-skill"
SKILL_NAME_DISAMBIGUATION_HASH_LENGTH: int = 8
